@@ -27,7 +27,7 @@
 #include	"md_u.h"
 #include	<dirent.h>
 
-int Detail(char *dev, int brief, int export, int test, char *homehost)
+int Detail(char *dev, int brief, int export, int test, char *homehost, char *prefer)
 {
 	/*
 	 * Print out details for an md array by using
@@ -58,7 +58,7 @@ int Detail(char *dev, int brief, int export, int test, char *homehost)
 
 	int rv = test ? 4 : 1;
 	int avail_disks = 0;
-	char *avail;
+	char *avail = NULL;
 
 	if (fd < 0) {
 		fprintf(stderr, Name ": cannot open %s: %s\n",
@@ -105,7 +105,7 @@ int Detail(char *dev, int brief, int export, int test, char *homehost)
 		int dn = st->container_dev;
 
 		member = subarray;
-		container = map_dev(dev2major(dn), dev2minor(dn), 1);
+		container = map_dev_preferred(dev2major(dn), dev2minor(dn), 1, prefer);
 	}
 
 	/* try to load a superblock */
@@ -367,16 +367,18 @@ int Detail(char *dev, int brief, int export, int test, char *homehost)
 			if (avail_disks == array.raid_disks)
 				st = "";
 			else if (!enough(array.level, array.raid_disks,
-					 array.layout, 1, avail, avail_disks))
+					 array.layout, 1, avail))
 				st = ", FAILED";
 			else
 				st = ", degraded";
 
-			printf("          State : %s%s%s%s\n",
-			       (array.state&(1<<MD_SB_CLEAN))?"clean":"active",
-			       st,
-			       (!e || e->percent < 0) ? "" : sync_action[e->resync],
-			       larray_size ? "": ", Not Started");
+			printf("          State : %s%s%s%s%s%s \n",
+			       (array.state&(1<<MD_SB_CLEAN))?"clean":"active", st,
+			       (!e || (e->percent < 0 && e->percent != PROCESS_PENDING &&
+			       e->percent != PROCESS_DELAYED)) ? "" : sync_action[e->resync],
+			       larray_size ? "": ", Not Started",
+			       e->percent == PROCESS_DELAYED ? " (DELAYED)": "",
+			       e->percent == PROCESS_PENDING ? " (PENDING)": "");
 		}
 		if (array.raid_disks)
 			printf(" Active Devices : %d\n", array.active_disks);
@@ -416,10 +418,8 @@ int Detail(char *dev, int brief, int export, int test, char *homehost)
 		}
 
 		if (e && e->percent >= 0) {
-			printf(" Re%s Status : %d%% complete\n",
-			       (st && st->sb && info->reshape_active)?
-			          "shape":"build",
-			       e->percent);
+			static char *sync_action[] = {"Rebuild", "Resync", "Reshape", "Check"};
+			printf(" %7s Status : %d%% complete\n", sync_action[e->resync], e->percent);
 			is_rebuilding = 1;
 		}
 		free_mdstat(ms);
@@ -430,12 +430,9 @@ This is pretty boring
 			printf("  Reshape pos'n : %llu%s\n", (unsigned long long) info->reshape_progress<<9,
 			       human_size((unsigned long long)info->reshape_progress<<9));
 #endif
-			if (info->delta_disks > 0)
+			if (info->delta_disks != 0)
 				printf("  Delta Devices : %d, (%d->%d)\n",
 				       info->delta_disks, array.raid_disks - info->delta_disks, array.raid_disks);
-			if (info->delta_disks < 0)
-				printf("  Delta Devices : %d, (%d->%d)\n",
-				       info->delta_disks, array.raid_disks, array.raid_disks + info->delta_disks);
 			if (info->new_level != array.level) {
 				char *c = map_num(pers, info->new_level);
 				printf("      New Level : %s\n", c?c:"-unknown-");
@@ -489,13 +486,14 @@ This is pretty boring
 				if (load_sys(path, vbuf) < 0)
 					continue;
 				if (strncmp(vbuf, "external:", 9) != 0 ||
-				    !is_subarray(sra->sys_name+9) ||
+				    !is_subarray(vbuf+9) ||
 				    strncmp(vbuf+10, sra->sys_name, nlen) != 0 ||
 				    vbuf[10+nlen] != '/')
 					continue;
 				dn = devname2devnum(de->d_name);
-				printf(" %s", map_dev(dev2major(dn),
-						      dev2minor(dn), 1));
+				printf(" %s", map_dev_preferred(
+					       dev2major(dn),
+					       dev2minor(dn), 1, prefer));
 			}
 			if (dir)
 				closedir(dir);
@@ -561,7 +559,7 @@ This is pretty boring
 		if (test && d < array.raid_disks
 		    && !(disk.state & (1<<MD_DISK_SYNC)))
 			rv |= 1;
-		if ((dv=map_dev(disk.major, disk.minor, 0))) {
+		if ((dv=map_dev_preferred(disk.major, disk.minor, 0, prefer))) {
 			if (brief) {
 				if (devices) {
 					devices = realloc(devices,
@@ -583,13 +581,15 @@ This is pretty boring
 	if (brief) printf("\n");
 	if (test &&
 	    !enough(array.level, array.raid_disks, array.layout,
-		    1, avail, avail_disks))
+		    1, avail))
 		rv = 2;
 
 	free(disks);
 out:
 	close(fd);
 	free(subarray);
+	free(avail);
+	sysfs_free(sra);
 	return rv;
 }
 
